@@ -42,81 +42,20 @@ static const struct grub_arg_option options[] =
 static const char grub_human_sizes[] = {' ', 'K', 'M', 'G', 'T'};
 
 static grub_err_t
-grub_ls_list_disks (int longlist)
+grub_ls_list_devices (int longlist)
 {
-  auto int grub_ls_print_disks (const char *name);
-  int grub_ls_print_disks (const char *name)
+  auto int grub_ls_print_devices (const char *name);
+  int grub_ls_print_devices (const char *name)
     {
-      grub_device_t dev;
-      auto int print_partition (const grub_partition_t p);
-      
-      int print_partition (const grub_partition_t p)
-	{
-	  char *pname = grub_partition_get_name (p);
-
-	  if (pname)
-	    {
-	      if (longlist)
-		grub_print_partinfo (dev, pname);
-	      else
-		grub_printf ("(%s,%s) ", name, pname);
-	    }
-
-	  return 0;
-	}
-      
-      dev = grub_device_open (name);
-      grub_errno = GRUB_ERR_NONE;
-      
-      if (dev)
-	{
-	  if (longlist)
-	    {
-	      grub_printf ("Device: %s", name);
-
-	      if (! dev->disk || ! dev->disk->has_partitions)
-		{
-		  grub_fs_t fs;
-		  char *label;
-
-		  fs = grub_fs_probe (dev);
-		  grub_errno = GRUB_ERR_NONE;
-
-		  grub_printf (", Filesystem type %s",
-			       fs ? fs->name : "unknown");
-
-		  if (fs && fs->label)
-		    {
-		      (fs->label) (dev, &label);
-		      if (grub_errno == GRUB_ERR_NONE)
-			{
-			  if (label && grub_strlen (label))
-			    grub_printf (", Label: %s", label);
-			  grub_free (label);
-			}
-		      else
-			grub_errno = GRUB_ERR_NONE;
-		    }
-		}
-
-	      grub_putchar ('\n');
-	    }
-	  else
-	    grub_printf ("(%s) ", name);
-
-	  if (dev->disk && dev->disk->has_partitions)
-	    {
-	      grub_partition_iterate (dev->disk, print_partition);
-	      grub_errno = GRUB_ERR_NONE;
-	    }
-
-	  grub_device_close (dev);
-	}
+      if (longlist)
+	grub_normal_print_device_info (name);
+      else
+	grub_printf ("(%s) ", name);
   
       return 0;
     }
   
-  grub_disk_dev_iterate (grub_ls_print_disks);
+  grub_device_iterate (grub_ls_print_devices);
   grub_putchar ('\n');
   grub_refresh ();
 
@@ -223,26 +162,8 @@ grub_ls_list_files (char *dirname, int longlist, int all, int human)
     {
       if (grub_errno == GRUB_ERR_UNKNOWN_FS)
 	grub_errno = GRUB_ERR_NONE;
-	  
-      grub_printf ("(%s): Filesystem is %s",
-		   device_name, fs ? fs->name : "unknown");
-      
-      if (fs && fs->label)
-	{
-	  char *label;
-	  
-	  (fs->label) (dev, &label);
-	  if (grub_errno == GRUB_ERR_NONE)
-	    {
-	      if (label && grub_strlen (label))
-		grub_printf (", Label: %s", label);
-	      grub_free (label);
-	    }
-	  else
-	    grub_errno = GRUB_ERR_NONE;
-	}
-      
-      grub_putchar ('\n');
+
+      grub_normal_print_device_info (device_name);
     }
   else if (fs)
     {
@@ -299,7 +220,7 @@ static grub_err_t
 grub_cmd_ls (struct grub_arg_list *state, int argc, char **args)
 {
   if (argc == 0)
-    grub_ls_list_disks (state[0].set);
+    grub_ls_list_devices (state[0].set);
   else
     grub_ls_list_files (args[0], state[0].set, state[2].set,
 			state[1].set);
